from typing import List, Tuple
from pandas import read_csv
import numpy
import os
from pypeepa.userInteraction.selectOptionQuestion import selectOptionQuestion
from pypeepa.userInteraction.printArray import printArray


def askHeaderForMultipleCSV(csv_list: List[str], csv_dir: str) -> List[Tuple[str, str]]:
    """Ask the user to select headers for many csv files, it will skip repeated sets of headers
    @param:`csv_list`:List of csv file names.
    @param:`csv_dir`:The directory of the csvs.
    @return: Returns a list of tuples containing the csv file name and the selected header name
    """
    files_and_header = []
    prev_cols = []
    col_index = None
    for csv_file in csv_list:
        csv_full_path = os.path.join(csv_dir, csv_file)
        current_columns = read_csv(
            csv_full_path, nrows=1, encoding_errors="ignore", encoding="cp437"
        ).columns
        if not numpy.array_equal(numpy.array(prev_cols), numpy.array(current_columns)):
            printArray(current_columns)
            col_index = selectOptionQuestion(
                question=f"Enter the index of the column you want to match.",
                min=1,
                max=len(current_columns),
            )
            prev_cols = current_columns
        col_name = current_columns[col_index - 1]
        files_and_header.append((csv_full_path, col_name))

    return files_and_header



from pypeepa.utils.loggingHandler import loggingHandler  # check
from typing import Callable, Any, Optional
from logging import Logger
import time


def measureTimeToRun(func: Callable[[Any], Any], logger: Optional[Logger] = None):
    """
    Times any function wrapped with it and if a logger is provided logs it.
    @param: `func`: the function to time.
    @param: `logger`: The Logger object
    @return:
        The result from the function or the None if no result
    """

    def wrapper(*args, **kwargs):
        tick = time.time()
        result = func(*args, **kwargs)
        loggingHandler(logger, f"Time taken for {func.__name__}: {time.time()-tick}")
        if result != None:
            return result

    return wrapper



from .askYNQuestion import askYNQuestion
from .printArray import printArray
from .selectOptionQuestion import selectOptionQuestion
from .signature import signature
from .getFilePath import getFilePath, sanitizeFilePath
from .progressBarIterator import progressBarIterator
from .askHeaderForMultipleCSV import askHeaderForMultipleCSV



import json
from typing import Optional, List, Any
from logging import Logger
from pypeepa.utils.loggingHandler import loggingHandler


def readJSON(file_name: str, logger: Optional[Logger] = None) -> List[Any]:
    """
    Reads a .json file and returns the result.\n
    @param: `file_name`: Name of the .json file\n
    @param: `loggingHandler`: (Optional)loggingHandler function from pypeepa.utils\n
    @return: Returns the json data in an array or empty array if any error occurs.\n
    """

    read_data = []

    try:
        with open(file_name, "r") as openfile:
            read_data = json.load(openfile)
    except FileNotFoundError:
        fnf_mssg = f"File '{file_name}' not found."
        loggingHandler(logger, fnf_mssg)
    except json.JSONDecodeError as e:
        jde_mssg = f"Error decoding JSON in '{file_name}': {e}"
        loggingHandler(logger, jde_mssg)
    except Exception as e:
        exc_mssg = f"An error occurred while reading '{file_name}': {e}"
        loggingHandler(logger, exc_mssg)

    return read_data
